@@ -7,14 +7,16 @@ from itertools import izip
 
 # Some convenience functions
     
-def make_sparse_labeled_tensor(ndim, labels=None, initial=None):
+def make_sparse_labeled_tensor(ndim, labels=None, initial=None, accumulate=None):
     from csc.divisi.tensor import DictTensor
     if labels is None: labels = [OrderedSet() for _ in xrange(ndim)]
     tensor = LabeledView(DictTensor(ndim), labels)
     if initial is not None:
         tensor.update(initial)
+    for k, v in accumulate or []:
+        tensor.inc(k, v)
     return tensor
-        
+
 
 def make_dense_labeled_tensor(data, labels):
     from csc.divisi.tensor import DenseTensor
@@ -43,8 +45,7 @@ class LabelingMixin(object):
     def row_label(self, index): return self.label(0, index)
     def column_label(self, index): return self.label(1, index)
     def labels(self, indices):
-        return tuple(self.label(mode, index)
-                     for mode, index in enumerate(indices))
+        return tuple(label_list[index] for label_list, index in izip(self._labels, indices))
     def label_lists(self):
         """
         A list of all label lists for this tensor, in order by mode.
@@ -76,6 +77,11 @@ class LabelingMixin(object):
         '''
         Look up the underlying tensor keys for a tuple of labels.
         '''
+        if not create:
+            try:
+                return tuple(label_list.index(label) for label_list, label in izip(self._labels, labels))
+            except TypeError: # happens for slices
+                pass
         return tuple(self.index(mode, label, create)
                      for mode, label in enumerate(labels))
 
@@ -224,6 +230,10 @@ class LabeledView(LabelingMixin, View):
         self.tensor += other
         return self
 
+    def icmul(self, other):
+        self.tensor *= other
+        return self
+
     def tensor_iadd(self, other):
         if not hasattr(other, 'label_list'):
             raise TypeError('Only labeled views can be added to labeled views.')
@@ -447,13 +457,13 @@ class LabeledSVD2DResults(SVD2DResults):
         root = fileh.root
         def wrapped_tensor(name):
             data = getattr(root, name)
-            if copy: data = np.array(data)
+            if copy: data = data.read()
             return DenseTensor(data)
         def wrapped_labeled_tensor(name, label_name):
             return wrapped_tensor(name).labeled(list(getattr(root, label_name)))
         u = wrapped_labeled_tensor('u', 'u_labels')
         v = wrapped_labeled_tensor('v', 'v_labels')
-        svals = DenseTensor(root.svals)
+        svals = root.svals.read()
         svd = cls(u, v, svals)
         svd._weighted_u = wrapped_tensor('weighted_u').labeled(u.label_lists())
         svd._weighted_v = wrapped_tensor('weighted_v').labeled(v.label_lists())
