@@ -6,15 +6,18 @@ from numpy import zeros
 from math import sqrt, log
 
 class BaseNormalizedView(View):
-    '''The base class for normalizing views, methods to be shared by all 3
-       should go here'''
+    '''
+    The base class for normalized views -- views that scale the values in a
+    tensor in some way.
+    '''
     # Specify that this view maps numeric labels to (the same) numeric labels.
     input_is_numeric = True
     output_is_numeric = True
 
     def __init__(self, tensor, mode=0):
-        '''tensor: tensor to view.
-           mode: which mode to normalize over (0 = rows, 1 = columns, etc.)
+        '''
+        * tensor: tensor to view.
+        * mode: which mode to normalize over (0 = rows, 1 = columns, etc.)
         '''
         if len(tensor) == 0:
             raise ValueError('Tensor ' + repr(tensor) + ' is empty.')
@@ -40,25 +43,37 @@ class BaseNormalizedView(View):
 
     @property
     def normalize_mode(self):
+        """
+        Which mode of the tensor is normalized (0 for rows, 1 for columns,
+        etc.)
+        """
         return self._normalize_mode
 
     @property
     def norms(self):
+        """
+        A list of the normalization factors that were computed for each
+        row/column/whatever.
+        """
         return self._norms
 
     def item_changed(self, indices, oldvalue, newvalue):
         self.update_norm(indices, oldvalue, newvalue)
 
     def refresh_norms():
+        """
+        Compute all of the normalization factors from scratch.
+        """
         import inspect
-	caller = inspect.getouterframes(inspect.currentframe())[1][3]
-	raise NotImplementedError(caller + ' must be implemented in subclass')
+        caller = inspect.getouterframes(inspect.currentframe())[1][3]
+        raise NotImplementedError(caller + ' must be implemented in subclass')
 
 
 class NormalizedView(BaseNormalizedView):
     '''A normalized view of a tensor.
 
-    The norm is a Euclidean norm (sqrt of sum of squares) over one dimension.
+    The norm is a Euclidean norm (sqrt of sum of squares) over one or
+    more dimensions.
 
     To demonstrate its use, let's create a demonstration ordinary tensor:
     >>> a = SparseLabeledTensor(ndim=2)
@@ -84,14 +99,20 @@ class NormalizedView(BaseNormalizedView):
     >>> an.base = 1
     >>> an['apple', 'red'] * sqrt(4)
     1.0
+
+    You can also supply a list of modes to normalize over.
     '''
     
+    @property
+    def modes(self):
+        modes = self._normalize_mode
+        if not isinstance(modes, (list, tuple)):
+            modes = [modes]
+        return modes
 
     def refresh_norms(self):
         tensor = self.tensor
-        mode = self._normalize_mode
-        if not isinstance(mode, list): modes = [mode]
-        else: modes = mode
+        modes = self.modes
         norms = [zeros((tensor.shape[mode],)) for mode in modes]
         for key, value in self.tensor.iteritems():
             for mode in modes:
@@ -99,13 +120,10 @@ class NormalizedView(BaseNormalizedView):
         self._norms = norms
 
     def update_norm(self, indices, prev, current):
-        mode = self._normalize_mode
-        if not isinstance(mode, list): modes = [mode]
-        else: modes = mode
+        modes = self.modes
         # Handle growing the norms array.
-        for mode in modes:
+        for i_mode, mode in enumerate(modes):
                 index = indices[mode]
-                i_mode = modes.index(mode)
                 while index >= self._norms[i_mode].shape[0]:
                     self._norms[i_mode].resize((2*self._norms[i_mode].shape[0],))
                 self._norms[i_mode][index] += (current*current - prev*prev)
@@ -113,16 +131,21 @@ class NormalizedView(BaseNormalizedView):
     def __getitem__(self, indices):
         if not isinstance(indices, (list, tuple)):
             indices = (indices,)
-        mode = self._normalize_mode
-        if not isinstance(mode, list): modes = [mode]
-        else: modes = mode
-        norm = self.base ** 2
-        for i, i_mode in [(indices[mode], modes.index(mode)) for mode in modes]:
-            norm += self._norms[i_mode][i]
+        modes = self.modes
+        norm = self.base ** 2 + sum(self._norms[i_mode][indices[mode]]
+                                    for i_mode, mode in enumerate(modes))
         data = self.tensor[indices]
         #if abs(data) < EPSILON and abs(norm) < EPSILON: return 0.0
         return data / sqrt(norm)
 
+    def iteritems(self):
+        modes = self._normalize_mode
+        if not isinstance(modes, (list, tuple)): modes = [modes]
+        base_squared = self.base ** 2
+        for indices, value in self.tensor.iteritems():
+            norm = base_squared + sum(self._norms[i_mode][indices[mode]]
+                                      for i_mode, mode in enumerate(modes))
+            yield indices, value / sqrt(norm)
 
     ### Extraction and views
     def normalized(self, mode=0):
@@ -133,17 +156,17 @@ class NormalizedView(BaseNormalizedView):
 
     def layer_on(self, tensor):
         # FIXME: This looks shady.
-        return NormalizedView(self.tensor, self.mode)
+        return NormalizedView(self.tensor, self.modes)
 
     def _svd(self, **kw):
-        if self.normalize_mode != 0:
-            raise NotImplementedError
-	return self.tensor._svd(row_factors=self.norms[0], **kw)
+        if self.normalize_mode == 0:
+            return self.tensor._svd(row_factors=self.norms[0], **kw)
+        return super(NormalizedView, self)._svd(**kw)
 
     def _isvd(self, **kw):
-        if self.normalize_mode != 0:
-	    raise NotImplementedError
-	return self.tensor._isvd(row_factors=self.norms[0], **kw)
+        if self.normalize_mode == 0:
+            return self.tensor._isvd(row_factors=self.norms[0], **kw)
+        return super(NormalizedView, self)._isvd(**kw)
     
     # All other operations fall back to the tensor.
 
@@ -221,14 +244,16 @@ class ZeroMeanView(View):
     (value of original key).
 
     For example, the tensor
-    0  1  2  4
-    0  0  3  0
+    
+        0  1  2  4
+        0  0  3  0
 
     mirrored along mode 0 is:
-    0  1  2  4 
-    0  0  3  0
-    0 -1 -2 -4
-    0  0 -3  0
+    
+        0  1  2  4 
+        0  0  3  0
+        0 -1 -2 -4
+        0  0 -3  0
     '''
     def __init__(self, tensor, mode):
         self.duplicate_mode = mode
