@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 class OrderedSet(object):
     """An OrderedSet acts very much like a list. There are two important
     differences:
@@ -81,8 +79,7 @@ class OrderedSet(object):
         '''Initialize a new OrderedSet.'''
         self.items = []     # list of all keys
         self.indices = {}   # maps known keys to their indices in the list
-        if origitems:
-            for item in origitems: self.add(item)
+        for item in origitems or []: self.add(item)
 
         self._setup_quick_lookup_methods()
 
@@ -167,6 +164,7 @@ class OrderedSet(object):
         # then deleted, it should be considered equal.
         
         # Get 'items' from the other thing, in case it's an OrderedSet.
+        if self is other: return True
         return self.items == getattr(other, 'items', other)
     def __ne__(self, other):
         return not self == other
@@ -174,13 +172,14 @@ class OrderedSet(object):
 
 class IdentitySet(object):
     '''
-    An object that behaves like an :class:OrderedSet, but simply contains
+    An object that behaves like an :class:`OrderedSet`, but simply contains
     the range of numbers up to *len*. Thus, every number is its own index.
         
     IdentitySets are used to label :class:`Tensors <divisi.tensor.Tensor>` on
     axes where labels would be meaningless or unnecessary.
     '''
     index_is_efficient = True
+    __slots__ = ['len']
     
     def __init__(self, len):
         self.len = len
@@ -202,7 +201,8 @@ class IdentitySet(object):
         return self.items == getattr(other, 'items', other)
     def __ne__(self, other): return not self == other
 
-
+    def __getstate__(self): return dict(len=self.len)
+    def __setstate__(self, state): self.len = state['len']
 
 def indexable_set(x, dim):
     if x is None:
@@ -211,7 +211,3 @@ def indexable_set(x, dim):
         return x
     return OrderedSet(x)
 
-    
-if __name__ == '__main__':
-    import doctest
-    doctest.testmod()
