@@ -1,8 +1,3 @@
-'''
-Processes for performing singular value decomposition on a matrix or tensor,
-and working with the results.
-'''
-
 from csc.divisi.tensor import Tensor, DenseTensor, outer_tuple_iterator, TheEmptySlice
 #from csc.divisi.labeled_tensor import DenseLabeledTensor
 from itertools import imap
@@ -17,7 +12,7 @@ import sys
 class ReconstructedTensor(Tensor):
     '''
     A ReconstructedTensor is a wrapper around a
-    [SVDResults](divisi.svd.SVDResults) object. The
+    :class:`SVDResults` object. The
     ReconstructedTensor behaves like the tensor constructed by
     multiplying together the U, Sigma and V matrices (reconstructing
     the original matrix from its SVD).
@@ -30,8 +25,8 @@ class ReconstructedTensor(Tensor):
     '''
     def __init__(self, svd_results):
         '''
-        Create a ReconstructedTensor from a
-        [SVDResults](divisi.svd.SVDResults) object.
+        The constructor creates a ReconstructedTensor from
+        a :class:`SVDResults` object.
         '''
         self.svd = svd_results
         self.a = svd_results.orig
@@ -102,9 +97,9 @@ class SVDResults(object):
 
 class SVD2DResults(object):
     '''
-    This class wraps the _U_, Sigma (_S_) and _V_ matrices that result from
-    computing an SVD. An SVD decomposes the matrix _A_ s.t. _A =
-    U*S*(V^T)_. Both _U_ and _V_ are orthonormal matrices, and _S_ is a
+    This class wraps the *U*, Sigma (*S*) and *V* matrices that result from
+    computing an SVD. An SVD decomposes the matrix *A* such that ``A =
+    U*S*(V^T)``. Both *U* and *V* are orthonormal matrices, and *S* is a
     diagonal matrix.
 
     This class also provides utility methods to make common
@@ -113,8 +108,8 @@ class SVD2DResults(object):
 
     def __init__(self, u, v, svals):
         '''
-        Construct an SVD2DResults object from the matrices created by
-        an SVD. Note that svals is a 1-D vector of sigma values from
+        The constructor makes an SVD2DResults object from the matrices created
+        by an SVD. Note that svals is a 1-D vector of sigma values from
         the SVD.
         '''
         self.u = u
@@ -135,7 +130,7 @@ class SVD2DResults(object):
     @property
     def core(self):
         '''
-        Get the core tensor. The core tensor is the diagonal _S_
+        Get the core tensor. The core tensor is the diagonal *S*
         matrix from an SVD.
         '''
         if self._core is None:
@@ -145,7 +140,7 @@ class SVD2DResults(object):
     @property
     def weighted_u(self):
         '''
-        Return _U * S_, the result of weighting each column of the _U_
+        Return ``U * S``, the result of weighting each column of the *U*
         matrix by its corresponding sigma value.
         '''
         if self._weighted_u is None:
@@ -155,7 +150,7 @@ class SVD2DResults(object):
     @property
     def weighted_v(self):
         '''
-        Return _V * S_, the result of weighting each column of the _V_
+        Return `V * S`, the result of weighting each column of the *V*
         matrix by its corresponding sigma value.
         '''
         if self._weighted_v is None:
@@ -164,15 +159,15 @@ class SVD2DResults(object):
 
     def u_dotproducts_with(self, vec):
         '''
-        Return the dot product of _vec_ with every weighted _u_
+        Return the dot product of *vec* with every weighted *u*
         vector.
         '''
         return self.weighted_u * vec
 
     def u_angles_to(self, vec):
         '''
-        Return the cosine of the angle between _vec_ and every weighted
-        _u_ vector. This value can be used as a similarity measure that
+        Return the cosine of the angle between *vec* and every weighted
+        *u* vector. This value can be used as a similarity measure that
         ranges from -1 to 1.
         '''
         u = self.weighted_u
@@ -187,9 +182,9 @@ class SVD2DResults(object):
 
     def weighted_u_vec(self, idx, default_zeros=False):
         '''
-        Get the weighted _u_ vector with key _idx_.
+        Get the weighted *u* vector with key *idx*.
 
-        If _idx_ is not present and default_zeros is true, returns a
+        If *idx* is not present and default_zeros is true, returns a
         vector of zeros.
         '''
         try:
@@ -200,15 +195,15 @@ class SVD2DResults(object):
 
     def v_dotproducts_with(self, vec):
         '''
-        Compute the dot product of _vec_ with every weighted _v_
+        Compute the dot product of *vec* with every weighted *v*
         vector.
         '''
         return self.weighted_v * vec
 
     def v_angles_to(self, vec):
         '''
-        Return the cosine of the angle between _vec_ and every weighted
-        _v_ vector. This value can be used as a similarity measure that
+        Return the cosine of the angle between *vec* and every weighted
+        *v* vector. This value can be used as a similarity measure that
         ranges from -1 to 1.
         '''
         v = self.weighted_v
@@ -223,9 +218,9 @@ class SVD2DResults(object):
 
     def weighted_v_vec(self, idx, default_zeros=False):
         '''
-        Get the weighted _v_ vector with key _idx_.
+        Get the weighted *v* vector with key *idx*.
 
-        If _idx_ is not present and default_zeros is true, returns a
+        If *idx* is not present and default_zeros is true, returns a
         vector of zeros.
         '''
         try:
@@ -236,8 +231,8 @@ class SVD2DResults(object):
 
     def get_ahat(self, indices):
         '''
-        Get the value of one entry of the reconstructed matrix (_A^hat
-        = U * S * V^T).
+        Get the value of one entry of the reconstructed matrix (``A^hat
+        = U * S * V^T``).
         '''
         i0, i1 = indices
         try:
@@ -247,17 +242,17 @@ class SVD2DResults(object):
 
     def u_similarity(self, idx1, idx2):
         '''
-        Get the similarity of two rows of the weighted _U_
+        Get the similarity of two rows of the weighted *U*
         matrix. Similarity is measured by the dot product of the two
-        _u_ vectors.
+        *u* vectors.
         '''
         return self.u[idx1, :]*self.core*self.u[idx2, :]
 
     def v_similarity(self, idx1, idx2):
         '''
-        Get the similarity of two rows of the weighted _V_
+        Get the similarity of two rows of the weighted *V*
         matrix. Similarity is measured by the dot product of the two
-        _v_ vectors.
+        *v* vectors.
         '''
         return self.v[idx1, :]*self.core*self.v[idx2, :]
 
@@ -319,8 +314,8 @@ class SVD2DResults(object):
     
     def summarize(self, k=None, u_only=False, screen_width=80, output=sys.stdout):
         '''
-        For each axis (up to the _k_th axis), print the items of the
-        _u_ and _v_ matrices with the largest components on the axis.
+        For each axis (up to the *k*-th axis), print the items of the
+        *u* and *v* matrices with the largest components on the axis.
         '''
         if k is None: k = len(self.svals)
         else: k = min(k, len(self.svals))
@@ -390,9 +385,9 @@ class SVD2DResults(object):
 class Reconstructed2DTensor(Tensor):
     '''
     This class wraps the results of a 2-D SVD (represented by 
-    a [SVD2DResults](divisi.svd.SVD2DResults.html) object) and 
-    behaves like the [Tensor](divisi.tensor.Tensor.html) created 
-    by multiplying together the _U_, _S_ and _V_ matrices of the 
+    an :class:`SVD2DResults` object) and 
+    behaves like the :class:`Tensor` created 
+    by multiplying together the *U*, *S* and *V* matrices of the 
     SVD result.
 
     Reconstructed2DTensors require much less storage space than the
@@ -403,8 +398,8 @@ class Reconstructed2DTensor(Tensor):
     ndim = 2
     def __init__(self, svd):
         '''
-        Create a Reconstructed2DTensor from a 
-        [SVD2DResults](divisi.svd.SVD2DResults.html) object.
+        The constructor creates a Reconstructed2DTensor from 
+        a :class:`SVD2DResults` object.
         '''
         self.svd = svd
         self.shape = svd.u.shape[0], svd.v.shape[0]
@@ -517,8 +512,8 @@ class DenormalizedReconstructed2DTensor(Reconstructed2DTensor):
         
 def svd_sparse(tensor, k=50):
     '''
-    Compute the SVD of sparse tensor _tensor_ using Lanczos' algorithm.
-    The optional parameter _k_ is the number of sigma values to retain 
+    Compute the SVD of sparse tensor *tensor* using Lanczos' algorithm.
+    The optional parameter *k* is the number of sigma values to retain 
     (default value is 50).
     '''
     if tensor.ndim < 2:
@@ -530,8 +525,8 @@ def svd_sparse(tensor, k=50):
 
 def svd_sparse_order2(tensor, k=50):
     '''
-    Compute the SVD of sparse 2-D tensor _tensor_. The optional 
-    parameter _k_ is the number of sigma values to retain (default 
+    Compute the SVD of sparse 2-D tensor *tensor*. The optional 
+    parameter *k* is the number of sigma values to retain (default 
     value is 50).
     '''
 
@@ -549,8 +544,8 @@ def svd_sparse_order2(tensor, k=50):
 
 def svd_sparse_orderN(tensor, k=50):
     '''
-    Compute the higher-order SVD of _tensor_. The optional
-    parameter _k_ is the number of sigma values to retain (default 
+    Compute the higher-order SVD of *tensor*. The optional
+    parameter *k* is the number of sigma values to retain (default 
     value is 50).
     '''
 
@@ -573,23 +568,23 @@ def svd_sparse_orderN(tensor, k=50):
 
 def incremental_svd(tensor, k=50, niter=100, lrate=.1, logging_interval=None):
     '''
-    Compute SVD of 2-D tensor _tensor_ using an incremental SVD algorithm. 
+    Compute SVD of 2-D tensor *tensor* using an incremental SVD algorithm. 
     This algorithm ignores unknown entries in the sparse tensor instead 
-    of treating them as 0s (like Lanczos' algorithm does). This means that
+    of treating them as zeros (like Lanczos' algorithm does). This means that
     the incremental SVD of a sparse matrix will not necessarily equal the
     Lanczos' SVD of the same matrix.
 
     The incremental SVD algorithm performs gradient descent to find the 
-    _U_, _V_ and _S_ matrices that minimize the Frobenius norm 
-    (Root Mean Squared Error) of the reconstructed matrix.
+    *U*, *V* and *S* matrices that minimize the
+    Root Mean Squared error of the reconstructed matrix.
 
-    The optional parameter _k_ is the number of sigma values to retain 
-    (default 50). The _lrate_ parameter controls the "learning rate," a 
+    The optional parameter *k* is the number of sigma values to retain 
+    (default 50). The *lrate* parameter controls the "learning rate," a 
     factor that changes the rate of gradient descent (default .1). A larger value 
     means the gradient descent will take larger steps toward its goal. 
-    Setting _lrate_ to a "large" value may cause the algorithm to diverge,
+    Setting *lrate* to a "large" value may cause the algorithm to diverge,
     and will cause a mysterious overflow error.
-    The _niter_ parameter controls the number of steps performed by the
+    The *niter* parameter controls the number of steps performed by the
     gradient descent process (default 100).
     '''
     assert(tensor.ndim == 2)
