@@ -19,6 +19,7 @@ from csc.divisi.exceptions import DimensionMismatch
 from csc.divisi._svdlib import svd, dictSvd, isvd, dictIsvd
 from warnings import warn
 import logging
+from itertools import izip
 
 __all__ = ['TheEmptySlice',
            'Tensor', 'DictTensor', 'DenseTensor', # Data storage
@@ -41,7 +42,7 @@ TheEmptySlice = slice(None)
 #  - iter_dim_keys()
 #  - inc
 
-class Tensor(object, MyDictMixin):
+class Tensor(MyDictMixin):
     """
     Tensors are the main type of object handled by Divisi. This is the base
     class for all Tensors.
@@ -248,6 +249,24 @@ class Tensor(object, MyDictMixin):
         '''
         return self * other
 
+    def __imul__(self, other):
+        """
+        Handle multiplication in-place (``*=``). See the documentation
+        for :meth:`__mul__`.
+        """
+        if isscalar(other):
+            return self.icmul(other)
+        else:
+            raise NotImplementedError, "Don't know how to *= for non-scalars."
+
+    def icmul(self, other):
+        '''
+        Multiplies this tensor in-place by a scalar.
+
+        The ``*=`` operator uses this method.
+        '''
+        raise NotImplementedError, "Don't know how to multiply by a constant in place for %s" % type(self)
+        
     def dot(self, other):
         """
         Get the product of two tensors, using matrix multiplication.
@@ -278,6 +297,16 @@ class Tensor(object, MyDictMixin):
         """
         return self * (1.0/other)
 
+    def __idiv__(self, other):
+        """
+        Performs in-place scalar division.
+        
+        It actually just multiplies by 1/*other*, so if you're clever enough
+        you might be able to make it divide by some other kind of object.
+        """
+        self *= (1.0/other)
+        return self
+
     def norm(self):
         '''
         Calculate the Frobenius (or Euclidean) norm of this tensor.
@@ -290,7 +319,7 @@ class Tensor(object, MyDictMixin):
         '''
         mag = 0.0
         for v in self.itervalues():
-            mag += v**2
+            mag += v*v
         return math.sqrt(mag)
 
     magnitude = norm
@@ -335,38 +364,58 @@ class Tensor(object, MyDictMixin):
     
 
     ## Summarization operations
+    def extreme_items(self, n_biggest=10, n_smallest=10):
+        '''
+        Extract the items at either extreme of the tensor. Returns
+        (biggest, smallest).
+
+        In many applications, we're not interested in all the values
+        in a particular tensor we calculated, just the ones with the
+        highest magnitude. *extremes* returns the *n_biggest* biggest
+        and *n_smallest* smallest values in a tensor, along with their
+        indices, in the form used by ``dict.items()``.
+        
+        If this is a vector (as it often is), note that the indices are going
+        to be tuples with one thing in them, for consistency.
+
+        TODO:
+        Somewhat like Python's ``sorted`` function, you can specify a
+        ``key``, e.g., abs, which is applied to the value before it is
+        compared. The output uses the original values.
+        '''
+        from heapq import heapreplace
+        from sys import maxint
+        biggest = [(-maxint,None)] * n_biggest
+        smallest = [(-maxint,None)] * n_smallest # maintains the negative of the smallest part
+
+        # optimized case: no key
+        for k, v in self.iteritems():
+            if n_biggest and v > biggest[0][0]: heapreplace(biggest, (v, k))
+            if n_smallest and -v > smallest[0][0]: heapreplace(smallest, (-v, k))
+            
+        biggest.sort()
+        smallest.sort(reverse=True)
+        biggest = [(k, v) for v, k in biggest]
+        smallest = [(k, -v) for v, k in smallest]
+
+        return biggest, smallest
+        
     def extremes(self, n=10, top_only=False):
         """
         Extract the interesting parts of this tensor.
         
-        In many applications, we're not interested in all the values in a
-        particular tensor we calculated, just the ones with the highest
-        magnitude. *extremes* returns the *n* highest and
-        *n* lowest values
-        in a tensor, along with their indices, in the form used by
-        ``dict.items()``.
-        
-        If this is a vector (as it often is), note that the indices are going
-        to be tuples with one thing in them, for consistency.
         """
-        from heapq import heapreplace
-        from sys import maxint
-        top = [(-maxint,None)] * n
-        bottom = [(-maxint,None)] * n # maintains the negative of the bottom part
+        n_biggest = n
+        if top_only: n_smallest = 0
+        else: n_smallest = n
 
-        for k, v in self.iteritems():
-            if v > top[0][0]: heapreplace(top, (v, k))
-            if -v > bottom[0][0]: heapreplace(bottom, (-v, k))
-
-        top.sort()
-        bottom.sort(reverse=True)
-        top = [(k, v) for v, k in top]
-        bottom = [(k, -v) for v, k in bottom]
+        biggest, smallest = self.extreme_items(n_biggest, n_smallest)
 
         if top_only:
-            return top
+            return biggest
         else:
-            return bottom + top
+            return smallest + biggest
+
 
     def show_extremes(self, n=10, output=sys.stdout):
         """
@@ -403,8 +452,8 @@ class Tensor(object, MyDictMixin):
 class InvalidLayeringException(StandardError):
     pass
 
-# List of attributes never to try to delegate to the tensor.
-attribute_blacklist = set(('_getAttributeNames', 'trait_names'))
+forward_to_tensor = set([
+        'shape', 'ndim', 'has_key', 'default_value'])
 
 class View(Tensor):
     """    
@@ -425,15 +474,19 @@ class View(Tensor):
         Create a new View wrapping a Tensor.
         """
         self.tensor = tensor
+        self.ndim = tensor.ndim
 
     def __repr__(self):
         return '<View of %s>' % repr(self.tensor)
 
     def __getattr__(self, name):
         '''Fall back to the tensor operation.'''
-        if name.startswith('_') or name in attribute_blacklist:
+        if name in forward_to_tensor:
+            attr = getattr(self.tensor, name)
+            #setattr(self, name, attr)
+            return attr
+        else:
             raise AttributeError(name)
-        return getattr(self.tensor, name)
 
     def __iter__(self):
         return self.tensor.__iter__()
@@ -563,25 +616,36 @@ class DictTensor(Tensor):
         return subcount(self._data, self.ndim-1)
 
     def __iter__(self):
-        def iter_helper(dict, num_nested, key_base):
+        def iter_helper(dct, num_nested, key_base):
             if num_nested == 1:
-                for x in dict.__iter__():
+                for x in dct.__iter__():
                     # TODO: is using append here too slow?
                     yield key_base + (x,)
             else:
-                for key, child_dict in dict.iteritems():
+                for key, child_dict in dct.iteritems():
                     for x in iter_helper(child_dict, num_nested - 1, key_base + (key,)):
                         yield x
         return iter_helper(self._data, self.ndim, ())
 
-    def iteritems(self):
-        def iter_helper(dict, num_nested, key_base):
+    def itervalues(self):
+        def iter_helper(dct, num_nested):
             if num_nested == 1:
-                for x, v in dict.iteritems():
+                for v in dct.itervalues():
+                    yield v
+            else:
+                for child_dict in dct.itervalues():
+                    for x in iter_helper(child_dict, num_nested - 1):
+                        yield x
+        return iter_helper(self._data, self.ndim)
+
+    def iteritems(self):
+        def iter_helper(dct, num_nested, key_base):
+            if num_nested == 1:
+                for x, v in dct.iteritems():
                     # TODO: is using append here too slow?
                     yield (key_base + (x,)), v
             else:
-                for key, child_dict in dict.iteritems():
+                for key, child_dict in dct.iteritems():
                     for x in iter_helper(child_dict, num_nested - 1, key_base + (key,)):
                         yield x
         return iter_helper(self._data, self.ndim, ())
@@ -844,7 +908,18 @@ class DictTensor(Tensor):
             if v:
                 self[k] -= v
         return self
-        
+
+    def icmul(self, other):
+        def subiter(dct, to_go):
+            if to_go == 0:
+                for k in dct:
+                    dct[k] *= other
+            else:
+                for d in dct.itervalues():
+                    subiter(d, to_go-1) 
+        subiter(self._data, self.ndim-1)
+        return self
+    
     def __neg__(self):
         '''
         Return a new tensor whose values are the negation of this
@@ -871,33 +946,34 @@ except ImportError:
         '''
         Generate all valid indices in a tensor of the given shape.
 
-        >>> sorted(outer_tuple_iterator((3,)))
+        >>> list(outer_tuple_iterator((3,)))
         [(0,), (1,), (2,)]
-        >>> sorted(outer_tuple_iterator((2,2)))
+        >>> list(outer_tuple_iterator((2,2)))
         [(0, 0), (0, 1), (1, 0), (1, 1)]
-        >>> sorted(outer_tuple_iterator(()))
+        >>> list(outer_tuple_iterator(()))
         []
         '''
         if len(shape) == 0: return
         idx = [0]*len(shape)
+        maxdim = len(shape) - 1
         while True:
             yield tuple(idx)
-            idx[0] += 1
+            idx[maxdim] += 1
             # Wraparound
-            cur_dim = 0
+            cur_dim = maxdim
             while idx[cur_dim] >= shape[cur_dim]:
                 idx[cur_dim] = 0
-                cur_dim += 1
-                try:
-                    idx[cur_dim] += 1
-                except IndexError:
-                    raise StopIteration
+                cur_dim -= 1
+                if cur_dim == -1: raise StopIteration
+                idx[cur_dim] += 1
 
 def data(tensor):
     """
     Get the chewy (NumPy|dictionary) center of the tensor, if at all possible.
     """
-    return getattr(tensor, '_data', tensor)
+    if hasattr(tensor, '_data'): return tensor._data
+    elif hasattr(tensor, 'tensor'): return data(tensor.tensor)
+    else: return tensor
 
 def wrap_unless_scalar(result):
     if isscalar(result): return result
@@ -929,7 +1005,11 @@ class DenseTensor(Tensor):
 
 
     def __getitem__(self, indices):
-        return wrap_unless_scalar(self._data[indices])
+        res = self._data[indices]
+        if isinstance(indices, (int, long)) or all(isinstance(idx, (int, long)) for idx in indices):
+            return res
+        else:
+            return DenseTensor(res)
 
     def __setitem__(self, indices, value):
         self._data[indices] = value
@@ -948,6 +1028,12 @@ class DenseTensor(Tensor):
     def __iter__(self):
         return outer_tuple_iterator(self.shape)
 
+    def itervalues(self):
+        return iter(numpy.reshape(self._data, (-1,)))
+    
+    def iteritems(self):
+        return izip(iter(self), self.itervalues())
+
     def iter_dim_keys(self, dim):
         assert(dim >= 0 and dim < self.ndim)
         return xrange(0, self.shape[dim])
@@ -977,6 +1063,10 @@ class DenseTensor(Tensor):
         self._data += data(other)
         return self
 
+    def icmul(self, other):
+        self._data *= other
+        return self
+
     def __neg__(self):
         return DenseTensor(-self._data)
 
