@@ -140,6 +140,17 @@ class DictTensorTest(unittest.TestCase):
                                 [7, 1],
                                 [None, 4]])
 
+    def testICmul(self):
+        t1 = tensor_from_nested_list([[1, 2], [3, 4]])
+        assertTensorEqual(t1, [[1, 2], [3, 4]])
+        t1 *= 2
+        assertTensorEqual(t1, [[2, 4], [6, 8]])
+        
+    def testICdiv(self):
+        t1 = tensor_from_nested_list([[2, 4], [6, 8]])
+        t1 /= 2
+        assertTensorEqual(t1, [[1, 2], [3, 4]])
+
     def testReprOfEmpty(self):
         repr(self.tensor)
         self.tensor.example_key()
@@ -668,7 +679,11 @@ normalize_testcase2 = [[1, 0],
 normalize_testcase3 = [[70, 0],
                        [3, 4]]
 
-
+def tensor_from_nested_list(data, tensor_cls=DictTensor):
+    t = tensor_cls(2)
+    t.update(nested_list_to_dict(data))
+    return t
+    
 
 class NormalizedSVD2DTest2(unittest.TestCase):
     def assertItemsEqual(self, t1, t2):
@@ -677,39 +692,29 @@ class NormalizedSVD2DTest2(unittest.TestCase):
         
 
     def test_svd(self):
-        t1 = DictTensor(2)
-        t1.update(nested_list_to_dict(normalize_testcase2))
+        '''Ensure SVDs of tensors match up after normalizing.'''
+        t1 = tensor_from_nested_list(normalize_testcase2)
         svd1 = t1.normalized().svd(k=1)
 
-        t2 = DictTensor(2)
-        t2.update(nested_list_to_dict(normalize_testcase3))
+        t2 = tensor_from_nested_list(normalize_testcase3)
         svd2 = t2.normalized().svd(k=1)
         self.assertItemsEqual(svd1.u, svd2.u)
         self.assertItemsEqual(svd1.svals, svd2.svals)
         self.assertItemsEqual(svd1.v, svd2.v)
 
+        # Now try transposing and normalizing over the other mode
+        t3 = t1.T
+        t4 = t2.T
+        svd3 = t3.normalized(mode=1).svd(k=1)
+        svd4 = t4.normalized(mode=1).svd(k=1)
+        self.assertItemsEqual(svd1.u, svd3.v)
+        self.assertItemsEqual(svd1.svals, svd3.svals)
+        self.assertItemsEqual(svd1.v, svd3.u)
 
-
-#from ipdb import post_mortem
-class DebugTestProgram(unittest.TestProgram):
-    def parseArgs(self, argv):
-        self.debug = '--debug' in argv
-        if self.debug:
-            argv.remove('--debug')
-        return super(DebugTestProgram, self).parseArgs(argv)
-    
-    def runTests(self):
-        if self.debug:
-            try:
-                return self.test.debug()
-            except Exception:
-                print "Entering post-mortem debugger."
-		import sys
-		tb = sys.exc_info()[2]
-                import pdb; pdb.post_mortem(tb)
-        else:
-            return super(DebugTestProgram, self).runTests()
+        self.assertItemsEqual(svd3.u, svd4.u)
+        self.assertItemsEqual(svd3.svals, svd4.svals)
+        self.assertItemsEqual(svd3.v, svd4.v)
+        
 
 if __name__ == '__main__':
-    DebugTestProgram()
-    #unittest.main()
+    unittest.main()
